/*
tofuproxy -- flexible HTTP/HTTPS proxy, TLS terminator, X.509 TOFU
             manager, WARC/geminispace browser
Copyright (C) 2021-2023 Sergey Matveev <stargrave@stargrave.org>

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, version 3 of the License.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package fifos

import (
	"log"
	"os"

	"go.stargrave.org/tofuproxy/caches"
)

func listSpies(p string) {
	for {
		fd, err := os.OpenFile(p, os.O_WRONLY|os.O_APPEND, os.FileMode(0666))
		if err != nil {
			log.Fatalln(err)
		}
		caches.SpiesM.RLock()
		for _, spy := range caches.Spies {
			if _, err = fd.WriteString(spy + "\n"); err != nil {
				break
			}
		}
		caches.SpiesM.RUnlock()
		fd.Close()
	}
}

func addSpy(p string) {
	for {
		hosts := make(map[string]struct{})
		for _, line := range readLinesFromFIFO(p) {
			hosts[line] = struct{}{}
		}
		for host := range hosts {
			log.Printf("%s: adding host %s\n", p, host)
		}
		caches.SpiesM.Lock()
		for _, spy := range caches.Spies {
			hosts[spy] = struct{}{}
		}
		caches.Spies = caches.Spies[:0]
		for host := range hosts {
			caches.Spies = append(caches.Spies, host)
		}
		caches.SpiesM.Unlock()
	}
}
