/*
tofuproxy -- flexible HTTP/HTTPS proxy, TLS terminator, X.509 TOFU
             manager, WARC/geminispace browser
Copyright (C) 2021-2023 Sergey Matveev <stargrave@stargrave.org>

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, version 3 of the License.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package warc

import "strings"

type Header map[string]string

func NewHeader() Header {
	return make(map[string]string)
}

func (h Header) Set(key, value string) {
	h[strings.ToLower(key)] = value
}

func (h Header) Get(key string) string {
	return h[strings.ToLower(key)]
}

func (h Header) Del(key string) {
	delete(h, strings.ToLower(key))
}

func (h Header) AddLine(line string) {
	parts := strings.SplitN(line, ":", 2)
	if len(parts) != 2 {
		return
	}
	h.Set(parts[0], strings.TrimSpace(parts[1]))
}
